searchData={"content_type":"text/markdown","items":[{"doc":"`ExArray.Array` is a module that provides Array-ish functionality to Elixir.\n\nThe `ExArray.Array` structure is useful when fast random access, by index, is needed. It can be dynamically grown \nand shrunk by removing or adding elements from any position in the structure. \n\nWhen passing indexes to any function in the module, negative integers are supported. Negative integers are treated as \nan offset from the end of the structure. For example, if the structure contains 5 elements, the index `-1` will return \nthe last element, `-2` will return the second to last element, and so on. If the negative index runs over the beginning\nof the structure, the appropriate failure condition will occur, either raises or an error tuple, depending on the \nmethod being called. In the example with 5 elements, the index `-6` will result in an error, as there is no element \nbefore the first element. When a positive index is passed, the index is treated as an offset from the beginning and \nthe appropriate failure condition will occur if the index runs over the end of the structure.","ref":"ExArray.html","title":"ExArray","type":"module"},{"doc":"`ExArray.Array` can be constructed in a number of ways. The most common is to use the `new/0` \nfunction to create an empty structure. Alternatively, you can use the `new/1` function to create a structure\ncontaining the elements in the given list.\n\n```elixir \niex> ExArray.Array.new()\n%ExArray.Array{length: 0, contents: %{}}\n\niex> ExArray.Array.new([1,2,3])\n%ExArray.Array{length: 3, contents: %{0 => 1, 1 => 2, 2 => 3}}\n```","ref":"ExArray.html#module-construction","title":"Construction - ExArray","type":"module"},{"doc":"Elements can be accessed by index using either the `get/2` or `get!/2` functions. The difference between these two \nfunctions is that `get!/2` will raise an error if the index is out of bounds, while `get/2` will return an error \ntuple. \n\n```elixir\niex> ExArray.Array.get(ExArray.Array.new([1, 2, 3]), 1)\n{:ok, 2}\n\niex> ExArray.Array.get(ExArray.Array.new([1, 2, 3]), 3)\n{:error, :out_of_bounds}\n\niex> ExArray.Array.get!(ExArray.Array.new([1, 2, 3]), 1)\n2\n\niex> ExArray.Array.get!(ExArray.Array.new([1, 2, 3]), 3)\n** (ArgumentError) Index 3 is out of bounds for length 3\n```","ref":"ExArray.html#module-accessing-elements","title":"Accessing elements - ExArray","type":"module"},{"doc":"Adding elements to the end of the structure is fast as no other elements need to be re-indexed. This is done using \nthe `add/2` function. The `add/2` function returns a new structure with the element added to the end. \n\nThis operation should always succeed\n\n```elixir\niex> ExArray.Array.add(ExArray.Array.new([1, 2, 3]), 4)\n%ExArray.Array{length: 4, contents: %{0 => 1, 1 => 2, 2 => 3, 3 => 4}}\n```\n\nAdding elements to the beginning or middle of the structure is slower as all elements after the index need to be \nre-indexed. This is done using `add_at/3` or `add_at!/3`. These functions return a new structure with the element added at the given index. If an element existed at the given index, it and all other elements after it will be shifted one to the right. \n\n```elixir \niex> ExArray.Array.add_at(ExArray.Array.new([1, 2, 3]), 1, 4) \n{:ok, %ExArray.Array{length: 4, contents: %{0 => 1, 1 => 4, 2 => 2, 3 => 3}}}\n\niex> ExArray.Array.add_at(ExArray.Array.new([1, 2, 3]), 4, 4)\n{:error, :out_of_bounds}\n\niex> ExArray.Array.add_at!(ExArray.Array.new([1, 2, 3]), 1, 4)\n%ExArray.Array{length: 4, contents: %{0 => 1, 1 => 4, 2 => 2, 3 => 3}}\n\niex> ExArray.Array.add_at!(ExArray.Array.new([1, 2, 3]), 4, 4)\n** (ArgumentError) Index 4 is out of bounds for length 3\n```","ref":"ExArray.html#module-adding-elements","title":"Adding Elements - ExArray","type":"module"},{"doc":"Removing elements from the end of the structure is fast as no other elements need to be re-indexed. This is done \nusing the `remove/1` function. The `remove/1` function returns a new structure with the last element removed. \n\nIf the `ExArray.Array` is empty, the `remove/1` function will return the original structure.\n\nThis operation should always succeed\n\n```elixir \niex> ExArray.Array.remove(ExArray.Array.new([1, 2, 3]))\n%ExArray.Array{length: 2, contents: %{0 => 1, 1 => 2}}\n\niex> ExArray.Array.remove(ExArray.Array.new())\n%ExArray.Array{length: 0, contents: %{}}\n```\n\nRemoving elements from the beginning or middle of the structure is slower as all elements after the index need to be\nre-indexed. This is done using `remove_at/2` or `remove_at!/2`. These functions return a new structure with the \nelement at the given index removed. All elements after it will be shifted one to the left. \n\n```elixir\niex> ExArray.Array.remove_at(ExArray.Array.new([1, 2, 3]), 1)\n{:ok, %ExArray.Array{length: 2, contents: %{0 => 1, 1 => 3}}}\n\niex> ExArray.Array.remove_at(ExArray.Array.new([1, 2, 3]), 4)\n{:error, :out_of_bounds}\n\niex> ExArray.Array.remove_at!(ExArray.Array.new([1, 2, 3]), 1)\n%ExArray.Array{length: 2, contents: %{0 => 1, 1 => 3}}\n\niex> ExArray.Array.remove_at!(ExArray.Array.new([1, 2, 3]), 4)\n** (ArgumentError) Index 4 is out of bounds for length 3\n```","ref":"ExArray.html#module-removing-elements","title":"Removing Elements - ExArray","type":"module"},{"doc":"`ExArray.Array` implements the `Collectable` protocol. This means that it can be used with `Enum.into` and `for` \nspecial forms\n\n```elixir\niex> Enum.into([1, 2, 3], ExArray.Array.new())\n%ExArray.Array{length: 3, contents: %{0 => 1, 1 => 2, 2 => 3}}\n\niex> for x <- [1, 2, 3], into: ExArray.Array.new(), do: x\n%ExArray.Array{length: 3, contents: %{0 => 1, 1 => 2, 2 => 3}}\n```","ref":"ExArray.html#module-collectable-operations","title":"Collectable Operations - ExArray","type":"module"},{"doc":"`ExArray.Array` implements the `Enumerable` protocol. This means that it can be used with the functions \nin the `Enum` module. \n\n```elixir \niex> ExArray.Array.new([1, 2, 3]) |> Enum.map(&(&1 * 2))\n[2, 4, 6]\n\niex> ExArray.Array.new([1, 2, 3]) |> Enum.reduce(&(&1 + &2))\n6\n\niex> ExArray.Array.new([1, 2, 3]) |> Enum.filter(&(&1 > 1))\n[2, 3]\n\niex> ExArray.Array.new([1, 2, 3]) |> Enum.at(1)\n2\n```","ref":"ExArray.html#module-enum-operations","title":"Enum Operations - ExArray","type":"module"},{"doc":"An `ExArray.Array` is a data structure that contains a collection of elements. \n\nElements are stored in a Map with their index as the key. This allows for fast \nrandom access to elements by index, but slower insertion and deletion operations, \nunless the element is being added or deleted to the end of the `Array`.","ref":"ExArray.Array.html","title":"ExArray.Array","type":"module"},{"doc":"Adds the given element to the end of the Array","ref":"ExArray.Array.html#add/2","title":"ExArray.Array.add/2","type":"function"},{"doc":"iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.add(4)\n    %ExArray.Array{length: 4, contents: %{0 => 1, 1 => 2, 2 => 3, 3 => 4}}","ref":"ExArray.Array.html#add/2-examples","title":"Examples - ExArray.Array.add/2","type":"function"},{"doc":"Adds the given element to the Array at the given index.\n\nIf the given index is greater than the length of the Array,\nan {:error, :out_of_bounds} tuple is returned. If the given index is negative, \nit will be evaluated as an offset from the end of the Array.\n\nIf an element currently exists at the given index, it, and all following\nelements, are shifted right.","ref":"ExArray.Array.html#add_at/3","title":"ExArray.Array.add_at/3","type":"function"},{"doc":"iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.add_at(1, 4)\n    {:ok, %ExArray.Array{length: 4, contents: %{0 => 1, 1 => 4, 2 => 2, 3 => 3}}}\n\n    iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.add_at(-2, 4)\n    {:ok, %ExArray.Array{length: 4, contents: %{0 => 1, 1 => 4, 2 => 2, 3 => 3}}}\n\n    iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.add_at(4, 4)\n    {:error, :out_of_bounds}","ref":"ExArray.Array.html#add_at/3-examples","title":"Examples - ExArray.Array.add_at/3","type":"function"},{"doc":"Adds the given element to the Array at the given index. \n\nIf the given index is greater than the length of the Array,\nan `ArgumentError` is raised. If the given index is negative, it will be evaluated\nas an offset from the end of the Array.\n\nIf an element currently exists at the given index, it, and all following \nelements, are shifted right.t()","ref":"ExArray.Array.html#add_at!/3","title":"ExArray.Array.add_at!/3","type":"function"},{"doc":"iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.add_at!(1, 4)\n    %ExArray.Array{length: 4, contents: %{0 => 1, 1 => 4, 2 => 2, 3 => 3}}\n\n    iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.add_at!(-2, 4)\n    %ExArray.Array{length: 4, contents: %{0 => 1, 1 => 4, 2 => 2, 3 => 3}}\n\n    iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.add_at!(4, 4)\n    ** (ArgumentError) Index 4 is out of bounds for length 3","ref":"ExArray.Array.html#add_at!/3-examples","title":"Examples - ExArray.Array.add_at!/3","type":"function"},{"doc":"Returns `{:ok, element}` if the given index is within the bounds of the Array, \notherwise returns `{:error, :out_of_bounds}`.\n\nIf the given index is negative, it will be evaluated as an offset from the end of the Array.","ref":"ExArray.Array.html#get/2","title":"ExArray.Array.get/2","type":"function"},{"doc":"iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.get(1)\n    {:ok, 2}\n\n    iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.get(-1)\n    {:ok, 3}\n\n    iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.get(4)\n    {:error, :out_of_bounds}","ref":"ExArray.Array.html#get/2-examples","title":"Examples - ExArray.Array.get/2","type":"function"},{"doc":"Returns the element at the given index if it exists, otherwise raises an `ArgumentError`\n\nIf the given index is negative, it will be evaluated as an offset from the end of the Array.","ref":"ExArray.Array.html#get!/2","title":"ExArray.Array.get!/2","type":"function"},{"doc":"iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.get!(1)\n    2\n\n    iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.get!(-1)\n    3\n\n    iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.get!(4)\n    ** (ArgumentError) Index 4 is out of bounds for length 3","ref":"ExArray.Array.html#get!/2-examples","title":"Examples - ExArray.Array.get!/2","type":"function"},{"doc":"Creates an `ExArray.Array` with a length of zero, containing no elements.","ref":"ExArray.Array.html#new/0","title":"ExArray.Array.new/0","type":"function"},{"doc":"iex> ExArray.Array.new()\n    %ExArray.Array{length: 0, contents: %{}}","ref":"ExArray.Array.html#new/0-examples","title":"Examples - ExArray.Array.new/0","type":"function"},{"doc":"Creates an `ExArray.Array` with the given list of elements.","ref":"ExArray.Array.html#new/1","title":"ExArray.Array.new/1","type":"function"},{"doc":"iex> ExArray.Array.new([1, 2, 3])\n    %ExArray.Array{length: 3, contents: %{0 => 1, 1 => 2, 2 => 3}}","ref":"ExArray.Array.html#new/1-examples","title":"Examples - ExArray.Array.new/1","type":"function"},{"doc":"Removes the last element from the Array and returns the resulting Array. \n\nIf the array is empty, the original Array is returned.","ref":"ExArray.Array.html#remove/1","title":"ExArray.Array.remove/1","type":"function"},{"doc":"iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.remove()\n    %ExArray.Array{length: 2, contents: %{0 => 1, 1 => 2}}\n\n    iex> ExArray.Array.new([]) |> ExArray.Array.remove()\n    %ExArray.Array{length: 0, contents: %{}}","ref":"ExArray.Array.html#remove/1-examples","title":"Examples - ExArray.Array.remove/1","type":"function"},{"doc":"Removes the element at the given index from the Array and returns the resulting Array. \n\nIf the given index is invalid, an {:error, :out_of_bounds} tuple is returned. If the given index is negative,\nit will be evaluated as an offset from the end of the Array.","ref":"ExArray.Array.html#remove_at/2","title":"ExArray.Array.remove_at/2","type":"function"},{"doc":"iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.remove_at(1)\n    {:ok, %ExArray.Array{length: 2, contents: %{0 => 1, 1 => 3}}}\n\n    iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.remove_at(-1)\n    {:ok, %ExArray.Array{length: 2, contents: %{0 => 1, 1 => 2}}}\n\n    iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.remove_at(4)\n    {:error, :out_of_bounds}","ref":"ExArray.Array.html#remove_at/2-examples","title":"Examples - ExArray.Array.remove_at/2","type":"function"},{"doc":"Removes the element at the given index from the Array and returns the resulting Array. \n\nIf the given index is invalid, an `ArgumentError` is raised. If the given index is negative,\nit will be evaluated as an offset from the end of the Array.","ref":"ExArray.Array.html#remove_at!/2","title":"ExArray.Array.remove_at!/2","type":"function"},{"doc":"iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.remove_at!(1)\n    %ExArray.Array{length: 2, contents: %{0 => 1, 1 => 3}}\n\n    iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.remove_at!(-1)\n    %ExArray.Array{length: 2, contents: %{0 => 1, 1 => 2}}\n\n    iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.remove_at!(4)\n    ** (ArgumentError) Index 4 is out of bounds for length 3","ref":"ExArray.Array.html#remove_at!/2-examples","title":"Examples - ExArray.Array.remove_at!/2","type":"function"},{"doc":"Returns a `List` of all the elements in the Array. Ordering is retained.","ref":"ExArray.Array.html#to_list/1","title":"ExArray.Array.to_list/1","type":"function"},{"doc":"iex> ExArray.Array.new([1, 2, 3]) |> ExArray.Array.to_list()\n    [1, 2, 3]","ref":"ExArray.Array.html#to_list/1-examples","title":"Examples - ExArray.Array.to_list/1","type":"function"},{"doc":"","ref":"ExArray.Array.html#t:t/0","title":"ExArray.Array.t/0","type":"type"}]}